@@ -21,6 +21,8 @@ module Loader
 # @api private
 #
 module ModuleLoaders
+  ENVIRONMENT = 'environment'.freeze
+
   def self.system_loader_from(parent_loader, loaders)
     # Puppet system may be installed in a fixed location via RPM, installed as a Gem, via source etc.
     # The only way to find this across the different ways puppet can be installed is
@@ -37,6 +39,15 @@ module ModuleLoaders
                                                        )
   end
 
+  def self.environment_loader_from(parent_loader, loaders, env_path)
+    ModuleLoaders::FileBased.new(parent_loader,
+      loaders,
+      ENVIRONMENT,
+      File.join(env_path, 'lib'),
+      ENVIRONMENT
+    )
+  end
+
   def self.module_loader_from(parent_loader, loaders, module_name, module_path)
     ModuleLoaders::FileBased.new(parent_loader,
                                                        loaders,
@@ -46,6 +57,15 @@ module ModuleLoaders
                                                        )
   end
 
+  def self.pcore_resource_type_loader_from(parent_loader, loaders, environment_path)
+    ModuleLoaders::FileBased.new(parent_loader,
+      loaders,
+      nil,
+      environment_path,
+      'pcore_resource_types'
+    )
+  end
+
   class AbstractPathBasedModuleLoader < BaseLoader
 
     # The name of the module, or nil, if this is a global "component"
@@ -66,6 +86,7 @@ module ModuleLoaders
 
     # Initialize a kind of ModuleLoader for one module
     # @param parent_loader [Loader] loader with higher priority
+    # @param loaders [Loaders] the container for this loader
     # @param module_name [String] the name of the module (non qualified name), may be nil for a global "component"
     # @param path [String] the path to the root of the module (semantics defined by subclass)
     # @param loader_name [String] a name that is used for human identification (useful when module_name is nil)
@@ -81,6 +102,7 @@ module ModuleLoaders
       unless (loadables - LOADABLE_KINDS).empty?
         raise ArgumentError, 'given loadables are not of supported loadable kind'
       end
+      loaders.add_loader_by_name(self)
     end
 
     def loadables
@@ -92,8 +114,10 @@ module ModuleLoaders
     # @return [Loader::NamedEntry, nil found/created entry, or nil if not found
     #
     def find(typed_name)
+      # This loader is tailored to only find entries in the current runtime
+      return nil unless typed_name.name_authority == Pcore::RUNTIME_NAME_AUTHORITY
+
       # Assume it is a global name, and that all parts of the name should be used when looking up
-      name_part_index = 0
       name_parts = typed_name.name_parts
 
       # Certain types and names can be disqualified up front
@@ -105,10 +129,6 @@ module ModuleLoaders
         # ok since such a "module" cannot have namespaced content).
         #
         return nil unless name_parts[0] == module_name
-
-        # Skip the first part of the name when computing the path since the path already contains the name of the
-        # module
-        name_part_index = 1
       else
         # The name is in the global name space.
 
@@ -116,6 +136,7 @@ module ModuleLoaders
         case typed_name.type
         when :function
         when :resource_type
+        when :resource_type_pp
         when :type
         else
           # anything else cannot possibly be in this module
@@ -129,7 +150,7 @@ module ModuleLoaders
       # Find the file to instantiate, and instantiate the entity if file is found
       origin = nil
       if (smart_path = smart_paths.effective_paths(typed_name.type).find do |sp|
-          origin = sp.effective_path(typed_name, name_part_index)
+          origin = sp.effective_path(typed_name, global? ? 0 : 1)
           existing_path(origin)
         end)
         value = smart_path.instantiator.create(self, typed_name, origin, get_contents(origin))
@@ -185,12 +206,20 @@ module ModuleLoaders
       raise NotImplementedError.new
     end
 
+    # Answers the question if this loader represents a global component (true for resource type loader and environment loader)
+    #
+    # @return [Boolean] `true` if this loader represents a global component
+    #
+    def global?
+      module_name.nil? || module_name == ENVIRONMENT
+    end
+
     # Produces the private loader for the module. If this module is not already resolved, this will trigger resolution
     #
     def private_loader
       # The system loader has a nil module_name and it does not have a private_loader as there are no functions
       # that can only by called by puppet runtime - if so, it acts as the private loader directly.
-      @private_loader ||= ((module_name.nil? && self) || @loaders.private_loader_for_module(module_name))
+      @private_loader ||= (global? ? self : @loaders.private_loader_for_module(module_name))
     end
   end
 
