@@ -15,11 +15,16 @@ class GlobalDataProvider < ConfiguredDataProvider
       # Hiera version 3 needs access to special scope variables
       scope = lookup_invocation.scope
       unless scope.is_a?(Hiera::Scope)
-        lookup_invocation = Invocation.new(
+        hiera_invocation = Invocation.new(
           Hiera::Scope.new(scope),
           lookup_invocation.override_values,
           lookup_invocation.default_values,
           lookup_invocation.explainer)
+
+        # Confine to global scope unless an environment data provider has been defined (same as for hiera_xxx functions)
+        adapter = lookup_invocation.lookup_adapter
+        hiera_invocation.set_global_only unless adapter.global_only? || adapter.has_environment_data_provider?(lookup_invocation)
+        lookup_invocation = hiera_invocation
       end
 
       merge = MergeStrategy.strategy(merge)
@@ -32,9 +37,17 @@ class GlobalDataProvider < ConfiguredDataProvider
       end
 
       value = super(key, lookup_invocation, merge)
-      if lookup_invocation.hiera_xxx_call? && (merge.is_a?(HashMergeStrategy) || merge.is_a?(DeepMergeStrategy))
-        # hiera_hash calls should error when found values are not hashes
-        Types::TypeAsserter.assert_instance_of('value', Types::PHashType::DEFAULT, value)
+      if lookup_invocation.hiera_xxx_call?
+        if merge.is_a?(HashMergeStrategy) || merge.is_a?(DeepMergeStrategy)
+          # hiera_hash calls should error when found values are not hashes
+          Types::TypeAsserter.assert_instance_of('value', Types::PHashType::DEFAULT, value)
+        end
+        if !key.segments.nil? && (merge.is_a?(HashMergeStrategy) || merge.is_a?(UniqueMergeStrategy))
+          strategy = merge.is_a?(HashMergeStrategy) ? 'hash' : 'array'
+
+          # Fail with old familiar message from Hiera 3
+          raise Puppet::DataBinding::LookupError, "Resolution type :#{strategy} is illegal when accessing values using dotted keys. Offending key was '#{key}'"
+        end
       end
       value
     else
@@ -45,7 +58,7 @@ class GlobalDataProvider < ConfiguredDataProvider
   protected
 
   def assert_config_version(config)
-    raise Puppet::DataBinding::LookupError, "#{config.name} cannot be used in the global layer" if config.version == 4
+    config.fail(Issues::HIERA_UNSUPPORTED_VERSION_IN_GLOBAL) if config.version == 4
     config
   end
 
