@@ -2522,7 +2522,7 @@ class PHashType < PCollectionType
       key_t = key_t.normalize(guard) unless key_t.nil?
       value_t = @value_type
       value_t = value_t.normalize(guard) unless value_t.nil?
-      @size_type.nil? && @key_type.equal?(key_t) && @value_type.equal?(value_t) ? self : PHashType.new(key_t, value_t, nil)
+      @size_type.nil? && @key_type.equal?(key_t) && @value_type.equal?(value_t) ? self : PHashType.new(key_t, value_t, @size_type)
     end
   end
 
@@ -2668,7 +2668,7 @@ class PVariantType < PAnyType
   # @return [PAnyType] the resulting type
   # @api public
   def self.maybe_create(types)
-    types = types.uniq
+    types = flatten_variants(types).uniq
     types.size == 1 ? types[0] : new(types)
   end
 
@@ -2713,24 +2713,15 @@ class PVariantType < PAnyType
 
       if types.size == 1
         types[0]
-      elsif types.any? { |t| t.is_a?(PUndefType) }
-        # Undef entry present. Use an OptionalType with a normalized Variant of all types that are not Undef
-        POptionalType.new(PVariantType.maybe_create(types.reject { |ot| ot.is_a?(PUndefType) }).normalize(guard)).normalize(guard)
+      elsif types.any? { |t| t.is_a?(PUndefType) || t.is_a?(POptionalType) }
+        # Undef entry present. Use an OptionalType with a normalized Variant without Undefs and Optional wrappers
+        POptionalType.new(PVariantType.maybe_create(types.reject { |t| t.is_a?(PUndefType) }.map { |t| t.is_a?(POptionalType) ? t.type : t })).normalize
       else
         # Merge all variants into this one
-        types = types.map do |t|
-          if t.is_a?(PVariantType)
-            modified = true
-            t.types
-          else
-            t
-          end
-        end
-        types.flatten! if modified
+        types = PVariantType.flatten_variants(types)
         size_before_merge = types.size
 
         types = swap_not_undefs(types)
-        types = swap_optionals(types)
         types = merge_enums(types)
         types = merge_patterns(types)
         types = merge_version_ranges(types)
@@ -2748,6 +2739,20 @@ class PVariantType < PAnyType
     end
   end
 
+  def self.flatten_variants(types)
+    modified = false
+    types = types.map do |t|
+      if t.is_a?(PVariantType)
+        modified = true
+        t.types
+      else
+        t
+      end
+    end
+    types.flatten! if modified
+    types
+  end
+
   def hash
     @types.hash
   end
@@ -2806,20 +2811,6 @@ class PVariantType < PAnyType
     end
   end
 
-  # @api private
-  def swap_optionals(array)
-    if array.size > 1
-      parts = array.partition {|t| t.is_a?(POptionalType) }
-      optionals = parts[0]
-      if optionals.size > 1
-        others = parts[1]
-        others <<  POptionalType.new(PVariantType.maybe_create(optionals.map { |optional| optional.type }).normalize)
-        array = others
-      end
-    end
-    array
-  end
-
   # @api private
   def swap_not_undefs(array)
     if array.size > 1
