@@ -21,6 +21,8 @@ Puppet::Type.type(:service).provide :smf, :parent => :base do
   commands :adm => "/usr/sbin/svcadm", :svcs => "/usr/bin/svcs"
   commands :svccfg => "/usr/sbin/svccfg"
 
+  has_feature :refreshable
+
   def setupservice
       if resource[:manifest]
         [command(:svcs), "-l", @resource[:name]]
@@ -64,7 +66,12 @@ Puppet::Type.type(:service).provide :smf, :parent => :base do
   end
 
   def restartcmd
-    [command(:adm), :restart, @resource[:name]]
+    if Puppet::Util::Package.versioncmp(Facter.value(:operatingsystemrelease), '11.2') >= 0
+      [command(:adm), :restart, "-s", @resource[:name]]
+    else
+      # Synchronous restart only supported in Solaris 11.2 and above
+      [command(:adm), :restart, @resource[:name]]
+    end
   end
 
   def startcmd
@@ -73,7 +80,7 @@ Puppet::Type.type(:service).provide :smf, :parent => :base do
     when :maintenance, :degraded
       [command(:adm), :clear, @resource[:name]]
     else
-      [command(:adm), :enable, "-s", @resource[:name]]
+      [command(:adm), :enable, "-rs", @resource[:name]]
     end
   end
 
@@ -128,8 +135,8 @@ Puppet::Type.type(:service).provide :smf, :parent => :base do
       states = service_states
       state = states[1] == "-" ? states[0] : states[1]
     rescue Puppet::ExecutionFailure
-      info "Could not get status on service #{self.name}"
-      return :stopped
+      debug "Could not get status on service #{self.name} #{$!}"
+      return :absent
     end
 
     case state
