@@ -1,7 +1,6 @@
 require 'puppet'
 require 'puppet/util/tagging'
 require 'puppet/parameter'
-require 'puppet/data_providers'
 
 # The simplest resource class.  Eventually it will function as the
 # base class for all resource-like behaviour.
@@ -31,7 +30,7 @@ class Puppet::Resource
   TYPE_NODE  = 'Node'.freeze
   TYPE_SITE  = 'Site'.freeze
 
-  def self.from_data_hash(data)
+  def self.from_data_hash(data, json_deserializer = nil)
     raise ArgumentError, "No resource type provided in serialized data" unless type = data['type']
     raise ArgumentError, "No resource title provided in serialized data" unless title = data['title']
 
@@ -41,6 +40,15 @@ class Puppet::Resource
       params.each { |param, value| resource[param] = value }
     end
 
+    if ext_params = data['ext_parameters']
+      raise Puppet::Error, 'Unable to deserialize non-Data type parameters unless a deserializer is provided' unless json_deserializer
+      reader = json_deserializer.reader
+      ext_params.each do |param, value|
+        reader.re_initialize(value)
+        resource[param] = json_deserializer.read
+      end
+    end
+
     if sensitive_parameters = data['sensitive_parameters']
       resource.sensitive_parameters = sensitive_parameters.map(&:to_sym)
     end
@@ -62,7 +70,7 @@ class Puppet::Resource
     "#{@type}[#{@title}]#{to_hash.inspect}"
   end
 
-  def to_data_hash
+  def to_data_hash(json_serializer = nil)
     data = ([:type, :title, :tags] + ATTRIBUTES).inject({}) do |hash, param|
       next hash unless value = self.send(param)
       hash[param.to_s] = value
@@ -71,23 +79,56 @@ class Puppet::Resource
 
     data["exported"] ||= false
 
-    params = self.to_hash.inject({}) do |hash, ary|
-      param, value = ary
-
+    ext_params = {}
+    params = {}
+    self.to_hash.each_pair do |param, value|
       # Don't duplicate the title as the namevar
-      next hash if param == namevar and value == title
-
-      hash[param] = Puppet::Resource.value_to_pson_data(value)
-      hash
+      unless param == namevar && value == title
+        value = Puppet::Resource.value_to_pson_data(value)
+        if is_json_type?(value)
+          params[param] = value
+        elsif json_serializer.nil?
+          Puppet.warning("Resource '#{to_s}' contains a #{value.class.name} value. It will be converted to the String '#{value}'")
+          params[param] = value
+        else
+          ext_params[param] = value
+        end
+      end
     end
 
-    data["parameters"] = params unless params.empty?
+    data['parameters'] = params unless params.empty?
+    unless ext_params.empty?
+      writer = json_serializer.writer
+      ext_params.each_key do |key|
+        writer.clear_io
+        json_serializer.write(ext_params[key])
+        writer.finish
+        ext_params[key] = writer.to_a
+      end
+      data['ext_parameters'] = ext_params
+    end
 
     data["sensitive_parameters"] = sensitive_parameters unless sensitive_parameters.empty?
 
     data
   end
 
+  # @param [Object] value The value to test
+  # @return [Boolean] `true` if the value can be represented as JSON
+  # @api private
+  def is_json_type?(value)
+    case value
+    when NilClass, TrueClass, FalseClass, Integer, Float, String
+      true
+    when Array
+      value.all? { |elem| is_json_type?(elem) }
+    when Hash
+      value.all? { |key, val| key.is_a?(String) && is_json_type?(val) }
+    else
+      false
+    end
+  end
+
   def self.value_to_pson_data(value)
     if value.is_a? Array
       value.map{|v| value_to_pson_data(v) }
@@ -252,7 +293,7 @@ class Puppet::Resource
       #   * a "classic" 3.x ruby plugin
       #   * a compatible implementation (e.g. loading from pcore metadata)
       #   * a resolved user defined type
-      # 
+      #
       # ...then, modify the parameters to the "old" (agent side compatible) way
       # of describing the type/title with string/symbols.
       #
@@ -319,7 +360,7 @@ class Puppet::Resource
   # A resource is a capability (instance) if its underlying type is a
   # capability type
   def is_capability?
-    resource_type and resource_type.is_capability?
+    !resource_type.nil? && resource_type.is_capability?
   end
 
   # Returns the value of the 'export' metaparam as an Array
